err = (require 'err').instance 'argon'
out = (require 'styout').instance 'argon'
_ = require 'underscore'

err.logByDefault = true

argon =
  ize: (args, schema) ->
    argsLen = args.length

    logSuffix = if schema.name? then " in call to <b>#{schema.name}" else ''

    if schema.requiredArgs?
      # check that the number of arguments is correct
      requiredArgsLen = schema.requiredArgs.length
      for requiredArg, index in schema.requiredArgs when index >= argsLen
        a = ['Missing required argument', requiredArg]
        throw err 'IllegalArgument', out.parse interp(a) + logSuffix
      if argsLen > requiredArgsLen + 1
        a = ['Expected', requiredArgsLen, 'to', requiredArgsLen + 1, 'arguments, but got', argsLen]
        throw err 'IllegalArgument', out.parse interp(a) + logSuffix

      # perform conversions
      for requiredArg, index in schema.requiredArgs
        if requiredArg[-2..-1] is 's)'
          args[index] = [requiredArg] unless requiredArg instanceof Array

    else # no required arguments
      requiredArgsLen = 0

    # get options and default callback from args
    givenOptions = defaultCallback = null
    if argsLen is requiredArgsLen + 1
      if typeof args[argsLen - 1] is 'function'
        defaultCallback = args[argsLen - 1]
      else
        givenOptions = args[argsLen - 1]

    # merge given options with defaults
    options = schema.defaultOptions ? {}
    events = schema.events ? []
    if givenOptions
      for key, val of givenOptions
        if key of options or key in events
          options[key] = val
        else
          out.warn "No such option <b>#{key}</b>"

    # extract callbacks from options
    callbacks = {}
    for event in events
      if event of options
        callbacks[event] = options[event]
        delete options[event]
      else if defaultCallback?
        callbacks[event] = defaultCallback
      else
        callbacks[event] = null

    requiredArgs = Array::slice.call args, 0, requiredArgsLen
    result = _.extend [requiredArgs..., {options, callbacks}]

root = window ? exports
_.extend root, argon

# utility functions
interp = (strings, useHtml = true) ->
  result = ''
  for s, index in strings
    if index % 2 is 0
      result += " #{s}"
    else
      result += if useHtml then " <b>#{s}</b>" else " #{s} "
  result